# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class VerificationAttemptsSummaryList(ListResource):

    def __init__(self, version):
        """
        Initialize the VerificationAttemptsSummaryList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryList
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryList
        """
        super(VerificationAttemptsSummaryList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a VerificationAttemptsSummaryContext

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        """
        return VerificationAttemptsSummaryContext(self._version, )

    def __call__(self):
        """
        Constructs a VerificationAttemptsSummaryContext

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        """
        return VerificationAttemptsSummaryContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.VerificationAttemptsSummaryList>'


class VerificationAttemptsSummaryPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the VerificationAttemptsSummaryPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryPage
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryPage
        """
        super(VerificationAttemptsSummaryPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of VerificationAttemptsSummaryInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        """
        return VerificationAttemptsSummaryInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.VerificationAttemptsSummaryPage>'


class VerificationAttemptsSummaryContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the VerificationAttemptsSummaryContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        """
        super(VerificationAttemptsSummaryContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Attempts/Summary'.format(**self._solution)

    def fetch(self, verify_service_sid=values.unset,
              date_created_after=values.unset, date_created_before=values.unset,
              country=values.unset, channel=values.unset,
              destination_prefix=values.unset):
        """
        Fetch the VerificationAttemptsSummaryInstance

        :param unicode verify_service_sid: Filter the verification attempts considered on the summary by verify service.
        :param datetime date_created_after: Consider verification attempts create after this date on the summary.
        :param datetime date_created_before: Consider verification attempts created before this date on the summary.
        :param unicode country: Filter verification attempts considered on the summary by destination country.
        :param VerificationAttemptsSummaryInstance.Channels channel: Filter verification attempts considered on the summary by communication channel.
        :param unicode destination_prefix: Filters the attempts considered on the summary by destination prefix.

        :returns: The fetched VerificationAttemptsSummaryInstance
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        """
        data = values.of({
            'VerifyServiceSid': verify_service_sid,
            'DateCreatedAfter': serialize.iso8601_datetime(date_created_after),
            'DateCreatedBefore': serialize.iso8601_datetime(date_created_before),
            'Country': country,
            'Channel': channel,
            'DestinationPrefix': destination_prefix,
        })

        payload = self._version.fetch(method='GET', uri=self._uri, params=data, )

        return VerificationAttemptsSummaryInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.VerificationAttemptsSummaryContext {}>'.format(context)


class VerificationAttemptsSummaryInstance(InstanceResource):

    class Channels(object):
        SMS = "sms"
        CALL = "call"
        EMAIL = "email"
        WHATSAPP = "whatsapp"

    def __init__(self, version, payload):
        """
        Initialize the VerificationAttemptsSummaryInstance

        :returns: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        """
        super(VerificationAttemptsSummaryInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'total_attempts': deserialize.integer(payload.get('total_attempts')),
            'total_converted': deserialize.integer(payload.get('total_converted')),
            'total_unconverted': deserialize.integer(payload.get('total_unconverted')),
            'conversion_rate_percentage': deserialize.decimal(payload.get('conversion_rate_percentage')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: VerificationAttemptsSummaryContext for this VerificationAttemptsSummaryInstance
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryContext
        """
        if self._context is None:
            self._context = VerificationAttemptsSummaryContext(self._version, )
        return self._context

    @property
    def total_attempts(self):
        """
        :returns: Total of attempts made.
        :rtype: unicode
        """
        return self._properties['total_attempts']

    @property
    def total_converted(self):
        """
        :returns: Total of attempts confirmed by the end user.
        :rtype: unicode
        """
        return self._properties['total_converted']

    @property
    def total_unconverted(self):
        """
        :returns: Total of attempts made that were not confirmed by the end user.
        :rtype: unicode
        """
        return self._properties['total_unconverted']

    @property
    def conversion_rate_percentage(self):
        """
        :returns: Percentage of the confirmed messages over the total.
        :rtype: unicode
        """
        return self._properties['conversion_rate_percentage']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self, verify_service_sid=values.unset,
              date_created_after=values.unset, date_created_before=values.unset,
              country=values.unset, channel=values.unset,
              destination_prefix=values.unset):
        """
        Fetch the VerificationAttemptsSummaryInstance

        :param unicode verify_service_sid: Filter the verification attempts considered on the summary by verify service.
        :param datetime date_created_after: Consider verification attempts create after this date on the summary.
        :param datetime date_created_before: Consider verification attempts created before this date on the summary.
        :param unicode country: Filter verification attempts considered on the summary by destination country.
        :param VerificationAttemptsSummaryInstance.Channels channel: Filter verification attempts considered on the summary by communication channel.
        :param unicode destination_prefix: Filters the attempts considered on the summary by destination prefix.

        :returns: The fetched VerificationAttemptsSummaryInstance
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryInstance
        """
        return self._proxy.fetch(
            verify_service_sid=verify_service_sid,
            date_created_after=date_created_after,
            date_created_before=date_created_before,
            country=country,
            channel=channel,
            destination_prefix=destination_prefix,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.VerificationAttemptsSummaryInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserDefinedMessageList(ListResource):

    def __init__(self, version, account_sid, call_sid):
        """
        Initialize the UserDefinedMessageList

        :param Version version: Version that contains the resource
        :param account_sid: Account SID.
        :param call_sid: Call SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageList
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageList
        """
        super(UserDefinedMessageList, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, }
        self._uri = '/Accounts/{account_sid}/Calls/{call_sid}/UserDefinedMessages.json'.format(**self._solution)

    def create(self, content, idempotency_key=values.unset):
        """
        Create the UserDefinedMessageInstance

        :param unicode content: The User Defined Message in the form of URL-encoded JSON string.
        :param unicode idempotency_key: A unique string value to identify API call. This should be a unique string value per API call and can be a randomly generated.

        :returns: The created UserDefinedMessageInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageInstance
        """
        data = values.of({'Content': content, 'IdempotencyKey': idempotency_key, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return UserDefinedMessageInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.UserDefinedMessageList>'


class UserDefinedMessagePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the UserDefinedMessagePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: Account SID.
        :param call_sid: Call SID.

        :returns: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessagePage
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessagePage
        """
        super(UserDefinedMessagePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserDefinedMessageInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageInstance
        """
        return UserDefinedMessageInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            call_sid=self._solution['call_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.UserDefinedMessagePage>'


class UserDefinedMessageInstance(InstanceResource):

    def __init__(self, version, payload, account_sid, call_sid):
        """
        Initialize the UserDefinedMessageInstance

        :returns: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageInstance
        :rtype: twilio.rest.api.v2010.account.call.user_defined_message.UserDefinedMessageInstance
        """
        super(UserDefinedMessageInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'call_sid': payload.get('call_sid'),
            'sid': payload.get('sid'),
            'date_created': deserialize.rfc2822_datetime(payload.get('date_created')),
        }

        # Context
        self._context = None
        self._solution = {'account_sid': account_sid, 'call_sid': call_sid, }

    @property
    def account_sid(self):
        """
        :returns: Account SID.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def call_sid(self):
        """
        :returns: Call SID.
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def sid(self):
        """
        :returns: User Defined Message SID.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def date_created(self):
        """
        :returns: The date this User Defined Message was created.
        :rtype: datetime
        """
        return self._properties['date_created']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.UserDefinedMessageInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class EsimProfileList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the EsimProfileList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileList
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileList
        """
        super(EsimProfileList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/ESimProfiles'.format(**self._solution)

    def create(self, callback_url=values.unset, callback_method=values.unset,
               eid=values.unset):
        """
        Create the EsimProfileInstance

        :param unicode callback_url: The URL we should call after we have sent when the status of the eSIM Profile changes
        :param unicode callback_method: The HTTP method we should use to call callback_url
        :param unicode eid: Identifier of the eUICC that will claim the eSIM Profile

        :returns: The created EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        """
        data = values.of({'CallbackUrl': callback_url, 'CallbackMethod': callback_method, 'Eid': eid, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return EsimProfileInstance(self._version, payload, )

    def stream(self, eid=values.unset, sim_sid=values.unset, status=values.unset,
               limit=None, page_size=None):
        """
        Streams EsimProfileInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode eid: List the eSIM Profiles that have been associated with an EId
        :param unicode sim_sid: Find the eSIM Profile resource related to a Sim resource by providing the SIM SID
        :param EsimProfileInstance.Status status: List the eSIM Profiles that are in a given status
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.esim_profile.EsimProfileInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(eid=eid, sim_sid=sim_sid, status=status, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, eid=values.unset, sim_sid=values.unset, status=values.unset,
             limit=None, page_size=None):
        """
        Lists EsimProfileInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode eid: List the eSIM Profiles that have been associated with an EId
        :param unicode sim_sid: Find the eSIM Profile resource related to a Sim resource by providing the SIM SID
        :param EsimProfileInstance.Status status: List the eSIM Profiles that are in a given status
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.esim_profile.EsimProfileInstance]
        """
        return list(self.stream(eid=eid, sim_sid=sim_sid, status=status, limit=limit, page_size=page_size, ))

    def page(self, eid=values.unset, sim_sid=values.unset, status=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of EsimProfileInstance records from the API.
        Request is executed immediately

        :param unicode eid: List the eSIM Profiles that have been associated with an EId
        :param unicode sim_sid: Find the eSIM Profile resource related to a Sim resource by providing the SIM SID
        :param EsimProfileInstance.Status status: List the eSIM Profiles that are in a given status
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfilePage
        """
        data = values.of({
            'Eid': eid,
            'SimSid': sim_sid,
            'Status': status,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return EsimProfilePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of EsimProfileInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfilePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return EsimProfilePage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a EsimProfileContext

        :param sid: The SID of the eSIM Profile resource to fetch

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        """
        return EsimProfileContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a EsimProfileContext

        :param sid: The SID of the eSIM Profile resource to fetch

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        """
        return EsimProfileContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.EsimProfileList>'


class EsimProfilePage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the EsimProfilePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfilePage
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfilePage
        """
        super(EsimProfilePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of EsimProfileInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        """
        return EsimProfileInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.EsimProfilePage>'


class EsimProfileContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, sid):
        """
        Initialize the EsimProfileContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the eSIM Profile resource to fetch

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        """
        super(EsimProfileContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/ESimProfiles/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the EsimProfileInstance

        :returns: The fetched EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return EsimProfileInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Supersim.V1.EsimProfileContext {}>'.format(context)


class EsimProfileInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Status(object):
        NEW = "new"
        RESERVING = "reserving"
        AVAILABLE = "available"
        DOWNLOADED = "downloaded"
        INSTALLED = "installed"
        FAILED = "failed"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the EsimProfileInstance

        :returns: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        """
        super(EsimProfileInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'iccid': payload.get('iccid'),
            'sim_sid': payload.get('sim_sid'),
            'status': payload.get('status'),
            'eid': payload.get('eid'),
            'smdp_plus_address': payload.get('smdp_plus_address'),
            'error_code': payload.get('error_code'),
            'error_message': payload.get('error_message'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: EsimProfileContext for this EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileContext
        """
        if self._context is None:
            self._context = EsimProfileContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account to which the eSIM Profile resource belongs
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def iccid(self):
        """
        :returns: The ICCID associated with the Sim resource
        :rtype: unicode
        """
        return self._properties['iccid']

    @property
    def sim_sid(self):
        """
        :returns: The SID of the Sim resource that this eSIM Profile controls
        :rtype: unicode
        """
        return self._properties['sim_sid']

    @property
    def status(self):
        """
        :returns: The status of the eSIM Profile
        :rtype: EsimProfileInstance.Status
        """
        return self._properties['status']

    @property
    def eid(self):
        """
        :returns: Identifier of the eUICC that can claim the eSIM Profile
        :rtype: unicode
        """
        return self._properties['eid']

    @property
    def smdp_plus_address(self):
        """
        :returns: Address of the SM-DP+ server from which the Profile will be downloaded
        :rtype: unicode
        """
        return self._properties['smdp_plus_address']

    @property
    def error_code(self):
        """
        :returns: Code indicating the failure if the download of the SIM Profile failed and the eSIM Profile is in `failed` state
        :rtype: unicode
        """
        return self._properties['error_code']

    @property
    def error_message(self):
        """
        :returns: Error message describing the failure if the download of the SIM Profile failed and the eSIM Profile is in `failed` state
        :rtype: unicode
        """
        return self._properties['error_message']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def url(self):
        """
        :returns: The absolute URL of the eSIM Profile resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the EsimProfileInstance

        :returns: The fetched EsimProfileInstance
        :rtype: twilio.rest.supersim.v1.esim_profile.EsimProfileInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Supersim.V1.EsimProfileInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.content.v1.content import ContentList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Content

        :returns: V1 version of Content
        :rtype: twilio.rest.content.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._contents = None

    @property
    def contents(self):
        """
        :rtype: twilio.rest.content.v1.content.ContentList
        """
        if self._contents is None:
            self._contents = ContentList(self)
        return self._contents

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Content.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class BalanceList(ListResource):

    def __init__(self, version, account_sid):
        """
        Initialize the BalanceList

        :param Version version: Version that contains the resource
        :param account_sid: Account Sid.

        :returns: twilio.rest.api.v2010.account.balance.BalanceList
        :rtype: twilio.rest.api.v2010.account.balance.BalanceList
        """
        super(BalanceList, self).__init__(version)

        # Path Solution
        self._solution = {'account_sid': account_sid, }
        self._uri = '/Accounts/{account_sid}/Balance.json'.format(**self._solution)

    def fetch(self):
        """
        Fetch the BalanceInstance

        :returns: The fetched BalanceInstance
        :rtype: twilio.rest.api.v2010.account.balance.BalanceInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return BalanceInstance(self._version, payload, account_sid=self._solution['account_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.BalanceList>'


class BalancePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the BalancePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: Account Sid.

        :returns: twilio.rest.api.v2010.account.balance.BalancePage
        :rtype: twilio.rest.api.v2010.account.balance.BalancePage
        """
        super(BalancePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of BalanceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.balance.BalanceInstance
        :rtype: twilio.rest.api.v2010.account.balance.BalanceInstance
        """
        return BalanceInstance(self._version, payload, account_sid=self._solution['account_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.BalancePage>'


class BalanceInstance(InstanceResource):

    def __init__(self, version, payload, account_sid):
        """
        Initialize the BalanceInstance

        :returns: twilio.rest.api.v2010.account.balance.BalanceInstance
        :rtype: twilio.rest.api.v2010.account.balance.BalanceInstance
        """
        super(BalanceInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'balance': payload.get('balance'),
            'currency': payload.get('currency'),
        }

        # Context
        self._context = None
        self._solution = {'account_sid': account_sid, }

    @property
    def account_sid(self):
        """
        :returns: Account Sid.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def balance(self):
        """
        :returns: Account balance
        :rtype: unicode
        """
        return self._properties['balance']

    @property
    def currency(self):
        """
        :returns: Currency units
        :rtype: unicode
        """
        return self._properties['currency']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.BalanceInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.flex_api.v1.interaction.interaction_channel import InteractionChannelList


class InteractionList(ListResource):

    def __init__(self, version):
        """
        Initialize the InteractionList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.flex_api.v1.interaction.InteractionList
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionList
        """
        super(InteractionList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Interactions'.format(**self._solution)

    def create(self, channel, routing):
        """
        Create the InteractionInstance

        :param dict channel: The Interaction's channel
        :param dict routing: The Interaction's routing logic

        :returns: The created InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionInstance
        """
        data = values.of({'Channel': serialize.object(channel), 'Routing': serialize.object(routing), })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return InteractionInstance(self._version, payload, )

    def get(self, sid):
        """
        Constructs a InteractionContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.flex_api.v1.interaction.InteractionContext
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionContext
        """
        return InteractionContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a InteractionContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.flex_api.v1.interaction.InteractionContext
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionContext
        """
        return InteractionContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionList>'


class InteractionPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the InteractionPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.flex_api.v1.interaction.InteractionPage
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionPage
        """
        super(InteractionPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InteractionInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.interaction.InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionInstance
        """
        return InteractionInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionPage>'


class InteractionContext(InstanceContext):

    def __init__(self, version, sid):
        """
        Initialize the InteractionContext

        :param Version version: Version that contains the resource
        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.flex_api.v1.interaction.InteractionContext
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionContext
        """
        super(InteractionContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Interactions/{sid}'.format(**self._solution)

        # Dependents
        self._channels = None

    def fetch(self):
        """
        Fetch the InteractionInstance

        :returns: The fetched InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return InteractionInstance(self._version, payload, sid=self._solution['sid'], )

    @property
    def channels(self):
        """
        Access the channels

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        """
        if self._channels is None:
            self._channels = InteractionChannelList(self._version, interaction_sid=self._solution['sid'], )
        return self._channels

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionContext {}>'.format(context)


class InteractionInstance(InstanceResource):

    def __init__(self, version, payload, sid=None):
        """
        Initialize the InteractionInstance

        :returns: twilio.rest.flex_api.v1.interaction.InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionInstance
        """
        super(InteractionInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'channel': payload.get('channel'),
            'routing': payload.get('routing'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: InteractionContext for this InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionContext
        """
        if self._context is None:
            self._context = InteractionContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def channel(self):
        """
        :returns: The Interaction's channel
        :rtype: dict
        """
        return self._properties['channel']

    @property
    def routing(self):
        """
        :returns: A JSON Object representing the routing rules for the Interaction Channel
        :rtype: dict
        """
        return self._properties['routing']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The links
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the InteractionInstance

        :returns: The fetched InteractionInstance
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionInstance
        """
        return self._proxy.fetch()

    @property
    def channels(self):
        """
        Access the channels

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        """
        return self._proxy.channels

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class TollfreeVerificationList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the TollfreeVerificationList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationList
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationList
        """
        super(TollfreeVerificationList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Tollfree/Verifications'.format(**self._solution)

    def stream(self, tollfree_phone_number_sid=values.unset, status=values.unset,
               limit=None, page_size=None):
        """
        Streams TollfreeVerificationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode tollfree_phone_number_sid: The SID of the Phone Number associated with the Tollfree Verification
        :param TollfreeVerificationInstance.Status status: The compliance status of the Tollfree Verification record.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            tollfree_phone_number_sid=tollfree_phone_number_sid,
            status=status,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, tollfree_phone_number_sid=values.unset, status=values.unset,
             limit=None, page_size=None):
        """
        Lists TollfreeVerificationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode tollfree_phone_number_sid: The SID of the Phone Number associated with the Tollfree Verification
        :param TollfreeVerificationInstance.Status status: The compliance status of the Tollfree Verification record.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance]
        """
        return list(self.stream(
            tollfree_phone_number_sid=tollfree_phone_number_sid,
            status=status,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, tollfree_phone_number_sid=values.unset, status=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of TollfreeVerificationInstance records from the API.
        Request is executed immediately

        :param unicode tollfree_phone_number_sid: The SID of the Phone Number associated with the Tollfree Verification
        :param TollfreeVerificationInstance.Status status: The compliance status of the Tollfree Verification record.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationPage
        """
        data = values.of({
            'TollfreePhoneNumberSid': tollfree_phone_number_sid,
            'Status': status,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return TollfreeVerificationPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of TollfreeVerificationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return TollfreeVerificationPage(self._version, response, self._solution)

    def create(self, business_name, business_website, notification_email,
               use_case_categories, use_case_summary, production_message_sample,
               opt_in_image_urls, opt_in_type, message_volume,
               tollfree_phone_number_sid, customer_profile_sid=values.unset,
               business_street_address=values.unset,
               business_street_address2=values.unset, business_city=values.unset,
               business_state_province_region=values.unset,
               business_postal_code=values.unset, business_country=values.unset,
               additional_information=values.unset,
               business_contact_first_name=values.unset,
               business_contact_last_name=values.unset,
               business_contact_email=values.unset,
               business_contact_phone=values.unset):
        """
        Create the TollfreeVerificationInstance

        :param unicode business_name: The name of the business or organization using the Tollfree number
        :param unicode business_website: The website of the business or organization using the Tollfree number
        :param unicode notification_email: The email address to receive the notification about the verification result.
        :param list[unicode] use_case_categories: The category of the use case for the Tollfree Number. List as many are applicable.
        :param unicode use_case_summary: Further explaination on how messaging is used by the business or organization
        :param unicode production_message_sample: An example of message content, i.e. a sample message
        :param list[unicode] opt_in_image_urls: Link to an image that shows the opt-in workflow. Multiple images allowed and must be a publicly hosted URL
        :param TollfreeVerificationInstance.OptInType opt_in_type: Describe how a user opts-in to text messages
        :param unicode message_volume: Estimate monthly volume of messages from the Tollfree Number
        :param unicode tollfree_phone_number_sid: The SID of the Phone Number associated with the Tollfree Verification
        :param unicode customer_profile_sid: Customer's Profile Bundle BundleSid
        :param unicode business_street_address: The address of the business or organization using the Tollfree number
        :param unicode business_street_address2: The address of the business or organization using the Tollfree number
        :param unicode business_city: The city of the business or organization using the Tollfree number
        :param unicode business_state_province_region: The state/province/region of the business or organization using the Tollfree number
        :param unicode business_postal_code: The postal code of the business or organization using the Tollfree number
        :param unicode business_country: The country of the business or organization using the Tollfree number
        :param unicode additional_information: Additional information to be provided for verification
        :param unicode business_contact_first_name: The first name of the contact for the business or organization using the Tollfree number
        :param unicode business_contact_last_name: The last name of the contact for the business or organization using the Tollfree number
        :param unicode business_contact_email: The email address of the contact for the business or organization using the Tollfree number
        :param unicode business_contact_phone: The phone number of the contact for the business or organization using the Tollfree number

        :returns: The created TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        """
        data = values.of({
            'BusinessName': business_name,
            'BusinessWebsite': business_website,
            'NotificationEmail': notification_email,
            'UseCaseCategories': serialize.map(use_case_categories, lambda e: e),
            'UseCaseSummary': use_case_summary,
            'ProductionMessageSample': production_message_sample,
            'OptInImageUrls': serialize.map(opt_in_image_urls, lambda e: e),
            'OptInType': opt_in_type,
            'MessageVolume': message_volume,
            'TollfreePhoneNumberSid': tollfree_phone_number_sid,
            'CustomerProfileSid': customer_profile_sid,
            'BusinessStreetAddress': business_street_address,
            'BusinessStreetAddress2': business_street_address2,
            'BusinessCity': business_city,
            'BusinessStateProvinceRegion': business_state_province_region,
            'BusinessPostalCode': business_postal_code,
            'BusinessCountry': business_country,
            'AdditionalInformation': additional_information,
            'BusinessContactFirstName': business_contact_first_name,
            'BusinessContactLastName': business_contact_last_name,
            'BusinessContactEmail': business_contact_email,
            'BusinessContactPhone': business_contact_phone,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return TollfreeVerificationInstance(self._version, payload, )

    def get(self, sid):
        """
        Constructs a TollfreeVerificationContext

        :param sid: Tollfree Verification Sid

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        """
        return TollfreeVerificationContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a TollfreeVerificationContext

        :param sid: Tollfree Verification Sid

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        """
        return TollfreeVerificationContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.TollfreeVerificationList>'


class TollfreeVerificationPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the TollfreeVerificationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationPage
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationPage
        """
        super(TollfreeVerificationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of TollfreeVerificationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        """
        return TollfreeVerificationInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1.TollfreeVerificationPage>'


class TollfreeVerificationContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, sid):
        """
        Initialize the TollfreeVerificationContext

        :param Version version: Version that contains the resource
        :param sid: Tollfree Verification Sid

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        """
        super(TollfreeVerificationContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Tollfree/Verifications/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the TollfreeVerificationInstance

        :returns: The fetched TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return TollfreeVerificationInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.TollfreeVerificationContext {}>'.format(context)


class TollfreeVerificationInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Status(object):
        PENDING_REVIEW = "PENDING_REVIEW"
        IN_REVIEW = "IN_REVIEW"
        TWILIO_APPROVED = "TWILIO_APPROVED"
        TWILIO_REJECTED = "TWILIO_REJECTED"

    class OptInType(object):
        VERBAL = "VERBAL"
        WEB_FORM = "WEB_FORM"
        PAPER_FORM = "PAPER_FORM"
        VIA_TEXT = "VIA_TEXT"
        MOBILE_QR_CODE = "MOBILE_QR_CODE"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the TollfreeVerificationInstance

        :returns: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        """
        super(TollfreeVerificationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'customer_profile_sid': payload.get('customer_profile_sid'),
            'trust_product_sid': payload.get('trust_product_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'regulated_item_sid': payload.get('regulated_item_sid'),
            'business_name': payload.get('business_name'),
            'business_street_address': payload.get('business_street_address'),
            'business_street_address2': payload.get('business_street_address2'),
            'business_city': payload.get('business_city'),
            'business_state_province_region': payload.get('business_state_province_region'),
            'business_postal_code': payload.get('business_postal_code'),
            'business_country': payload.get('business_country'),
            'business_website': payload.get('business_website'),
            'business_contact_first_name': payload.get('business_contact_first_name'),
            'business_contact_last_name': payload.get('business_contact_last_name'),
            'business_contact_email': payload.get('business_contact_email'),
            'business_contact_phone': payload.get('business_contact_phone'),
            'notification_email': payload.get('notification_email'),
            'use_case_categories': payload.get('use_case_categories'),
            'use_case_summary': payload.get('use_case_summary'),
            'production_message_sample': payload.get('production_message_sample'),
            'opt_in_image_urls': payload.get('opt_in_image_urls'),
            'opt_in_type': payload.get('opt_in_type'),
            'message_volume': payload.get('message_volume'),
            'additional_information': payload.get('additional_information'),
            'tollfree_phone_number_sid': payload.get('tollfree_phone_number_sid'),
            'status': payload.get('status'),
            'url': payload.get('url'),
            'resource_links': payload.get('resource_links'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: TollfreeVerificationContext for this TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationContext
        """
        if self._context is None:
            self._context = TollfreeVerificationContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: Tollfree Verification Sid
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def customer_profile_sid(self):
        """
        :returns: Customer's Profile Bundle BundleSid
        :rtype: unicode
        """
        return self._properties['customer_profile_sid']

    @property
    def trust_product_sid(self):
        """
        :returns: Tollfree TrustProduct Bundle BundleSid
        :rtype: unicode
        """
        return self._properties['trust_product_sid']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def regulated_item_sid(self):
        """
        :returns: The SID of the Regulated Item
        :rtype: unicode
        """
        return self._properties['regulated_item_sid']

    @property
    def business_name(self):
        """
        :returns: The name of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_name']

    @property
    def business_street_address(self):
        """
        :returns: The address of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_street_address']

    @property
    def business_street_address2(self):
        """
        :returns: The address of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_street_address2']

    @property
    def business_city(self):
        """
        :returns: The city of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_city']

    @property
    def business_state_province_region(self):
        """
        :returns: The state/province/region of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_state_province_region']

    @property
    def business_postal_code(self):
        """
        :returns: The postal code of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_postal_code']

    @property
    def business_country(self):
        """
        :returns: The country of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_country']

    @property
    def business_website(self):
        """
        :returns: The website of the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_website']

    @property
    def business_contact_first_name(self):
        """
        :returns: The first name of the contact for the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_contact_first_name']

    @property
    def business_contact_last_name(self):
        """
        :returns: The last name of the contact for the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_contact_last_name']

    @property
    def business_contact_email(self):
        """
        :returns: The email address of the contact for the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_contact_email']

    @property
    def business_contact_phone(self):
        """
        :returns: The phone number of the contact for the business or organization using the Tollfree number
        :rtype: unicode
        """
        return self._properties['business_contact_phone']

    @property
    def notification_email(self):
        """
        :returns: The email address to receive the notification about the verification result.
        :rtype: unicode
        """
        return self._properties['notification_email']

    @property
    def use_case_categories(self):
        """
        :returns: The category of the use case for the Tollfree Number. List as many are applicable.
        :rtype: list[unicode]
        """
        return self._properties['use_case_categories']

    @property
    def use_case_summary(self):
        """
        :returns: Further explaination on how messaging is used by the business or organization
        :rtype: unicode
        """
        return self._properties['use_case_summary']

    @property
    def production_message_sample(self):
        """
        :returns: An example of message content, i.e. a sample message
        :rtype: unicode
        """
        return self._properties['production_message_sample']

    @property
    def opt_in_image_urls(self):
        """
        :returns: Link to an image that shows the opt-in workflow. Multiple images allowed and must be a publicly hosted URL
        :rtype: list[unicode]
        """
        return self._properties['opt_in_image_urls']

    @property
    def opt_in_type(self):
        """
        :returns: Describe how a user opts-in to text messages
        :rtype: TollfreeVerificationInstance.OptInType
        """
        return self._properties['opt_in_type']

    @property
    def message_volume(self):
        """
        :returns: Estimate monthly volume of messages from the Tollfree Number
        :rtype: unicode
        """
        return self._properties['message_volume']

    @property
    def additional_information(self):
        """
        :returns: Additional information to be provided for verification
        :rtype: unicode
        """
        return self._properties['additional_information']

    @property
    def tollfree_phone_number_sid(self):
        """
        :returns: The SID of the Phone Number associated with the Tollfree Verification
        :rtype: unicode
        """
        return self._properties['tollfree_phone_number_sid']

    @property
    def status(self):
        """
        :returns: The compliance status of the Tollfree Verification record.
        :rtype: TollfreeVerificationInstance.Status
        """
        return self._properties['status']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Tollfree Verification
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def resource_links(self):
        """
        :returns: The URLs of the documents associated with the Tollfree Verification resource
        :rtype: dict
        """
        return self._properties['resource_links']

    def fetch(self):
        """
        Fetch the TollfreeVerificationInstance

        :returns: The fetched TollfreeVerificationInstance
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Messaging.V1.TollfreeVerificationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class CallSummaryList(ListResource):

    def __init__(self, version, call_sid):
        """
        Initialize the CallSummaryList

        :param Version version: Version that contains the resource
        :param call_sid: The call_sid

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryList
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryList
        """
        super(CallSummaryList, self).__init__(version)

        # Path Solution
        self._solution = {'call_sid': call_sid, }

    def get(self):
        """
        Constructs a CallSummaryContext

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryContext
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryContext
        """
        return CallSummaryContext(self._version, call_sid=self._solution['call_sid'], )

    def __call__(self):
        """
        Constructs a CallSummaryContext

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryContext
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryContext
        """
        return CallSummaryContext(self._version, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallSummaryList>'


class CallSummaryPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the CallSummaryPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param call_sid: The call_sid

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryPage
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryPage
        """
        super(CallSummaryPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of CallSummaryInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        """
        return CallSummaryInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.CallSummaryPage>'


class CallSummaryContext(InstanceContext):

    def __init__(self, version, call_sid):
        """
        Initialize the CallSummaryContext

        :param Version version: Version that contains the resource
        :param call_sid: The call_sid

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryContext
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryContext
        """
        super(CallSummaryContext, self).__init__(version)

        # Path Solution
        self._solution = {'call_sid': call_sid, }
        self._uri = '/Voice/{call_sid}/Summary'.format(**self._solution)

    def fetch(self, processing_state=values.unset):
        """
        Fetch the CallSummaryInstance

        :param CallSummaryInstance.ProcessingState processing_state: The processing_state

        :returns: The fetched CallSummaryInstance
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        """
        data = values.of({'ProcessingState': processing_state, })

        payload = self._version.fetch(method='GET', uri=self._uri, params=data, )

        return CallSummaryInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.CallSummaryContext {}>'.format(context)


class CallSummaryInstance(InstanceResource):

    class AnsweredBy(object):
        UNKNOWN = "unknown"
        MACHINE_START = "machine_start"
        MACHINE_END_BEEP = "machine_end_beep"
        MACHINE_END_SILENCE = "machine_end_silence"
        MACHINE_END_OTHER = "machine_end_other"
        HUMAN = "human"
        FAX = "fax"

    class CallType(object):
        CARRIER = "carrier"
        SIP = "sip"
        TRUNKING = "trunking"
        CLIENT = "client"

    class CallState(object):
        RINGING = "ringing"
        COMPLETED = "completed"
        BUSY = "busy"
        FAIL = "fail"
        NOANSWER = "noanswer"
        CANCELED = "canceled"
        ANSWERED = "answered"
        UNDIALED = "undialed"

    class ProcessingState(object):
        COMPLETE = "complete"
        PARTIAL = "partial"

    def __init__(self, version, payload, call_sid):
        """
        Initialize the CallSummaryInstance

        :returns: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        """
        super(CallSummaryInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'call_sid': payload.get('call_sid'),
            'call_type': payload.get('call_type'),
            'call_state': payload.get('call_state'),
            'answered_by': payload.get('answered_by'),
            'processing_state': payload.get('processing_state'),
            'created_time': deserialize.iso8601_datetime(payload.get('created_time')),
            'start_time': deserialize.iso8601_datetime(payload.get('start_time')),
            'end_time': deserialize.iso8601_datetime(payload.get('end_time')),
            'duration': deserialize.integer(payload.get('duration')),
            'connect_duration': deserialize.integer(payload.get('connect_duration')),
            'from_': payload.get('from'),
            'to': payload.get('to'),
            'carrier_edge': payload.get('carrier_edge'),
            'client_edge': payload.get('client_edge'),
            'sdk_edge': payload.get('sdk_edge'),
            'sip_edge': payload.get('sip_edge'),
            'tags': payload.get('tags'),
            'url': payload.get('url'),
            'attributes': payload.get('attributes'),
            'properties': payload.get('properties'),
            'trust': payload.get('trust'),
            'annotation': payload.get('annotation'),
        }

        # Context
        self._context = None
        self._solution = {'call_sid': call_sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: CallSummaryContext for this CallSummaryInstance
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryContext
        """
        if self._context is None:
            self._context = CallSummaryContext(self._version, call_sid=self._solution['call_sid'], )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def call_sid(self):
        """
        :returns: The call_sid
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def call_type(self):
        """
        :returns: The call_type
        :rtype: CallSummaryInstance.CallType
        """
        return self._properties['call_type']

    @property
    def call_state(self):
        """
        :returns: The call_state
        :rtype: CallSummaryInstance.CallState
        """
        return self._properties['call_state']

    @property
    def answered_by(self):
        """
        :returns: The answered_by
        :rtype: CallSummaryInstance.AnsweredBy
        """
        return self._properties['answered_by']

    @property
    def processing_state(self):
        """
        :returns: The processing_state
        :rtype: CallSummaryInstance.ProcessingState
        """
        return self._properties['processing_state']

    @property
    def created_time(self):
        """
        :returns: The created_time
        :rtype: datetime
        """
        return self._properties['created_time']

    @property
    def start_time(self):
        """
        :returns: The start_time
        :rtype: datetime
        """
        return self._properties['start_time']

    @property
    def end_time(self):
        """
        :returns: The end_time
        :rtype: datetime
        """
        return self._properties['end_time']

    @property
    def duration(self):
        """
        :returns: The duration
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def connect_duration(self):
        """
        :returns: The connect_duration
        :rtype: unicode
        """
        return self._properties['connect_duration']

    @property
    def from_(self):
        """
        :returns: The from
        :rtype: dict
        """
        return self._properties['from_']

    @property
    def to(self):
        """
        :returns: The to
        :rtype: dict
        """
        return self._properties['to']

    @property
    def carrier_edge(self):
        """
        :returns: The carrier_edge
        :rtype: dict
        """
        return self._properties['carrier_edge']

    @property
    def client_edge(self):
        """
        :returns: The client_edge
        :rtype: dict
        """
        return self._properties['client_edge']

    @property
    def sdk_edge(self):
        """
        :returns: The sdk_edge
        :rtype: dict
        """
        return self._properties['sdk_edge']

    @property
    def sip_edge(self):
        """
        :returns: The sip_edge
        :rtype: dict
        """
        return self._properties['sip_edge']

    @property
    def tags(self):
        """
        :returns: The tags
        :rtype: list[unicode]
        """
        return self._properties['tags']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def attributes(self):
        """
        :returns: The attributes
        :rtype: dict
        """
        return self._properties['attributes']

    @property
    def properties(self):
        """
        :returns: The properties
        :rtype: dict
        """
        return self._properties['properties']

    @property
    def trust(self):
        """
        :returns: The trust
        :rtype: dict
        """
        return self._properties['trust']

    @property
    def annotation(self):
        """
        :returns: The annotation
        :rtype: dict
        """
        return self._properties['annotation']

    def fetch(self, processing_state=values.unset):
        """
        Fetch the CallSummaryInstance

        :param CallSummaryInstance.ProcessingState processing_state: The processing_state

        :returns: The fetched CallSummaryInstance
        :rtype: twilio.rest.insights.v1.call.summary.CallSummaryInstance
        """
        return self._proxy.fetch(processing_state=processing_state, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.CallSummaryInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class MediaRecordingList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the MediaRecordingList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingList
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingList
        """
        super(MediaRecordingList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/MediaRecordings'.format(**self._solution)

    def stream(self, order=values.unset, status=values.unset,
               processor_sid=values.unset, source_sid=values.unset, limit=None,
               page_size=None):
        """
        Streams MediaRecordingInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param MediaRecordingInstance.Order order: The sort order of the list
        :param MediaRecordingInstance.Status status: Status to filter by
        :param unicode processor_sid: MediaProcessor to filter by
        :param unicode source_sid: Source SID to filter by
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            order=order,
            status=status,
            processor_sid=processor_sid,
            source_sid=source_sid,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, order=values.unset, status=values.unset,
             processor_sid=values.unset, source_sid=values.unset, limit=None,
             page_size=None):
        """
        Lists MediaRecordingInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param MediaRecordingInstance.Order order: The sort order of the list
        :param MediaRecordingInstance.Status status: Status to filter by
        :param unicode processor_sid: MediaProcessor to filter by
        :param unicode source_sid: Source SID to filter by
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.media.v1.media_recording.MediaRecordingInstance]
        """
        return list(self.stream(
            order=order,
            status=status,
            processor_sid=processor_sid,
            source_sid=source_sid,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, order=values.unset, status=values.unset,
             processor_sid=values.unset, source_sid=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param MediaRecordingInstance.Order order: The sort order of the list
        :param MediaRecordingInstance.Status status: Status to filter by
        :param unicode processor_sid: MediaProcessor to filter by
        :param unicode source_sid: Source SID to filter by
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        data = values.of({
            'Order': order,
            'Status': status,
            'ProcessorSid': processor_sid,
            'SourceSid': source_sid,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return MediaRecordingPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of MediaRecordingInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return MediaRecordingPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a MediaRecordingContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        return MediaRecordingContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a MediaRecordingContext

        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        return MediaRecordingContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Media.V1.MediaRecordingList>'


class MediaRecordingPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the MediaRecordingPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingPage
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingPage
        """
        super(MediaRecordingPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of MediaRecordingInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        return MediaRecordingInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Media.V1.MediaRecordingPage>'


class MediaRecordingContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, sid):
        """
        Initialize the MediaRecordingContext

        :param Version version: Version that contains the resource
        :param sid: The SID that identifies the resource to fetch

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingContext
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        super(MediaRecordingContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/MediaRecordings/{sid}'.format(**self._solution)

    def delete(self):
        """
        Deletes the MediaRecordingInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def fetch(self):
        """
        Fetch the MediaRecordingInstance

        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return MediaRecordingInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Media.V1.MediaRecordingContext {}>'.format(context)


class MediaRecordingInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    class Format(object):
        MP4 = "mp4"
        WEBM = "webm"

    class Order(object):
        ASC = "asc"
        DESC = "desc"

    class Status(object):
        PROCESSING = "processing"
        COMPLETED = "completed"
        DELETED = "deleted"
        FAILED = "failed"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the MediaRecordingInstance

        :returns: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        super(MediaRecordingInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'duration': deserialize.integer(payload.get('duration')),
            'format': payload.get('format'),
            'links': payload.get('links'),
            'processor_sid': payload.get('processor_sid'),
            'resolution': payload.get('resolution'),
            'source_sid': payload.get('source_sid'),
            'sid': payload.get('sid'),
            'media_size': deserialize.integer(payload.get('media_size')),
            'status': payload.get('status'),
            'status_callback': payload.get('status_callback'),
            'status_callback_method': payload.get('status_callback_method'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: MediaRecordingContext for this MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingContext
        """
        if self._context is None:
            self._context = MediaRecordingContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def duration(self):
        """
        :returns: The duration of the MediaRecording
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def format(self):
        """
        :returns: The format of the MediaRecording
        :rtype: MediaRecordingInstance.Format
        """
        return self._properties['format']

    @property
    def links(self):
        """
        :returns: The URLs of related resources
        :rtype: unicode
        """
        return self._properties['links']

    @property
    def processor_sid(self):
        """
        :returns: The SID of the MediaProcessor
        :rtype: unicode
        """
        return self._properties['processor_sid']

    @property
    def resolution(self):
        """
        :returns: The dimensions of the video image in pixels
        :rtype: unicode
        """
        return self._properties['resolution']

    @property
    def source_sid(self):
        """
        :returns: The SID of the resource that generated the original media
        :rtype: unicode
        """
        return self._properties['source_sid']

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def media_size(self):
        """
        :returns: The size of the recording media
        :rtype: unicode
        """
        return self._properties['media_size']

    @property
    def status(self):
        """
        :returns: The status of the MediaRecording
        :rtype: MediaRecordingInstance.Status
        """
        return self._properties['status']

    @property
    def status_callback(self):
        """
        :returns: The URL to which Twilio will send MediaRecording event updates
        :rtype: unicode
        """
        return self._properties['status_callback']

    @property
    def status_callback_method(self):
        """
        :returns: The HTTP method Twilio should use to call the `status_callback` URL
        :rtype: unicode
        """
        return self._properties['status_callback_method']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    def delete(self):
        """
        Deletes the MediaRecordingInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def fetch(self):
        """
        Fetch the MediaRecordingInstance

        :returns: The fetched MediaRecordingInstance
        :rtype: twilio.rest.media.v1.media_recording.MediaRecordingInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Media.V1.MediaRecordingInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class IpAddressList(ListResource):

    def __init__(self, version, account_sid, ip_access_control_list_sid):
        """
        Initialize the IpAddressList

        :param Version version: Version that contains the resource
        :param account_sid: The unique id of the Account that is responsible for this resource.
        :param ip_access_control_list_sid: The unique id of the IpAccessControlList resource that includes this resource.

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressList
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressList
        """
        super(IpAddressList, self).__init__(version)

        # Path Solution
        self._solution = {
            'account_sid': account_sid,
            'ip_access_control_list_sid': ip_access_control_list_sid,
        }
        self._uri = '/Accounts/{account_sid}/SIP/IpAccessControlLists/{ip_access_control_list_sid}/IpAddresses.json'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams IpAddressInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists IpAddressInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of IpAddressInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return IpAddressPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of IpAddressInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return IpAddressPage(self._version, response, self._solution)

    def create(self, friendly_name, ip_address, cidr_prefix_length=values.unset):
        """
        Create the IpAddressInstance

        :param unicode friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param unicode ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param unicode cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The created IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'IpAddress': ip_address,
            'CidrPrefixLength': cidr_prefix_length,
        })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
        )

    def get(self, sid):
        """
        Constructs a IpAddressContext

        :param sid: A string that identifies the IpAddress resource to fetch

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        """
        return IpAddressContext(
            self._version,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a IpAddressContext

        :param sid: A string that identifies the IpAddress resource to fetch

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        """
        return IpAddressContext(
            self._version,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.IpAddressList>'


class IpAddressPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the IpAddressPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param account_sid: The unique id of the Account that is responsible for this resource.
        :param ip_access_control_list_sid: The unique id of the IpAccessControlList resource that includes this resource.

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressPage
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressPage
        """
        super(IpAddressPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of IpAddressInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Api.V2010.IpAddressPage>'


class IpAddressContext(InstanceContext):

    def __init__(self, version, account_sid, ip_access_control_list_sid, sid):
        """
        Initialize the IpAddressContext

        :param Version version: Version that contains the resource
        :param account_sid: The unique sid that identifies this account
        :param ip_access_control_list_sid: The IpAccessControlList Sid that identifies the IpAddress resources to fetch
        :param sid: A string that identifies the IpAddress resource to fetch

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        """
        super(IpAddressContext, self).__init__(version)

        # Path Solution
        self._solution = {
            'account_sid': account_sid,
            'ip_access_control_list_sid': ip_access_control_list_sid,
            'sid': sid,
        }
        self._uri = '/Accounts/{account_sid}/SIP/IpAccessControlLists/{ip_access_control_list_sid}/IpAddresses/{sid}.json'.format(**self._solution)

    def fetch(self):
        """
        Fetch the IpAddressInstance

        :returns: The fetched IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
            sid=self._solution['sid'],
        )

    def update(self, ip_address=values.unset, friendly_name=values.unset,
               cidr_prefix_length=values.unset):
        """
        Update the IpAddressInstance

        :param unicode ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param unicode friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param unicode cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        data = values.of({
            'IpAddress': ip_address,
            'FriendlyName': friendly_name,
            'CidrPrefixLength': cidr_prefix_length,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return IpAddressInstance(
            self._version,
            payload,
            account_sid=self._solution['account_sid'],
            ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
            sid=self._solution['sid'],
        )

    def delete(self):
        """
        Deletes the IpAddressInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.IpAddressContext {}>'.format(context)


class IpAddressInstance(InstanceResource):

    def __init__(self, version, payload, account_sid, ip_access_control_list_sid,
                 sid=None):
        """
        Initialize the IpAddressInstance

        :returns: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        super(IpAddressInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'ip_address': payload.get('ip_address'),
            'cidr_prefix_length': deserialize.integer(payload.get('cidr_prefix_length')),
            'ip_access_control_list_sid': payload.get('ip_access_control_list_sid'),
            'date_created': deserialize.rfc2822_datetime(payload.get('date_created')),
            'date_updated': deserialize.rfc2822_datetime(payload.get('date_updated')),
            'uri': payload.get('uri'),
        }

        # Context
        self._context = None
        self._solution = {
            'account_sid': account_sid,
            'ip_access_control_list_sid': ip_access_control_list_sid,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: IpAddressContext for this IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressContext
        """
        if self._context is None:
            self._context = IpAddressContext(
                self._version,
                account_sid=self._solution['account_sid'],
                ip_access_control_list_sid=self._solution['ip_access_control_list_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: A 34 character string that uniquely identifies this resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The unique id of the Account that is responsible for this resource.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: A human readable descriptive text for this resource, up to 255 characters long.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def ip_address(self):
        """
        :returns: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :rtype: unicode
        """
        return self._properties['ip_address']

    @property
    def cidr_prefix_length(self):
        """
        :returns: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.
        :rtype: unicode
        """
        return self._properties['cidr_prefix_length']

    @property
    def ip_access_control_list_sid(self):
        """
        :returns: The unique id of the IpAccessControlList resource that includes this resource.
        :rtype: unicode
        """
        return self._properties['ip_access_control_list_sid']

    @property
    def date_created(self):
        """
        :returns: The date that this resource was created, given as GMT in RFC 2822 format.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that this resource was last updated, given as GMT in RFC 2822 format.
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def uri(self):
        """
        :returns: The URI for this resource, relative to https://api.twilio.com
        :rtype: unicode
        """
        return self._properties['uri']

    def fetch(self):
        """
        Fetch the IpAddressInstance

        :returns: The fetched IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        return self._proxy.fetch()

    def update(self, ip_address=values.unset, friendly_name=values.unset,
               cidr_prefix_length=values.unset):
        """
        Update the IpAddressInstance

        :param unicode ip_address: An IP address in dotted decimal notation from which you want to accept traffic. Any SIP requests from this IP address will be allowed by Twilio. IPv4 only supported today.
        :param unicode friendly_name: A human readable descriptive text for this resource, up to 255 characters long.
        :param unicode cidr_prefix_length: An integer representing the length of the CIDR prefix to use with this IP address when accepting traffic. By default the entire IP address is used.

        :returns: The updated IpAddressInstance
        :rtype: twilio.rest.api.v2010.account.sip.ip_access_control_list.ip_address.IpAddressInstance
        """
        return self._proxy.update(
            ip_address=ip_address,
            friendly_name=friendly_name,
            cidr_prefix_length=cidr_prefix_length,
        )

    def delete(self):
        """
        Deletes the IpAddressInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Api.V2010.IpAddressInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class EventList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, call_sid):
        """
        Initialize the EventList

        :param Version version: Version that contains the resource
        :param call_sid: The call_sid

        :returns: twilio.rest.insights.v1.call.event.EventList
        :rtype: twilio.rest.insights.v1.call.event.EventList
        """
        super(EventList, self).__init__(version)

        # Path Solution
        self._solution = {'call_sid': call_sid, }
        self._uri = '/Voice/{call_sid}/Events'.format(**self._solution)

    def stream(self, edge=values.unset, limit=None, page_size=None):
        """
        Streams EventInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param EventInstance.TwilioEdge edge: The edge
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.call.event.EventInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(edge=edge, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, edge=values.unset, limit=None, page_size=None):
        """
        Lists EventInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param EventInstance.TwilioEdge edge: The edge
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.call.event.EventInstance]
        """
        return list(self.stream(edge=edge, limit=limit, page_size=page_size, ))

    def page(self, edge=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of EventInstance records from the API.
        Request is executed immediately

        :param EventInstance.TwilioEdge edge: The edge
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of EventInstance
        :rtype: twilio.rest.insights.v1.call.event.EventPage
        """
        data = values.of({
            'Edge': edge,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return EventPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of EventInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of EventInstance
        :rtype: twilio.rest.insights.v1.call.event.EventPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return EventPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.EventList>'


class EventPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the EventPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param call_sid: The call_sid

        :returns: twilio.rest.insights.v1.call.event.EventPage
        :rtype: twilio.rest.insights.v1.call.event.EventPage
        """
        super(EventPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of EventInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.call.event.EventInstance
        :rtype: twilio.rest.insights.v1.call.event.EventInstance
        """
        return EventInstance(self._version, payload, call_sid=self._solution['call_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.EventPage>'


class EventInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    class TwilioEdge(object):
        UNKNOWN_EDGE = "unknown_edge"
        CARRIER_EDGE = "carrier_edge"
        SIP_EDGE = "sip_edge"
        SDK_EDGE = "sdk_edge"
        CLIENT_EDGE = "client_edge"

    class Level(object):
        UNKNOWN = "UNKNOWN"
        DEBUG = "DEBUG"
        INFO = "INFO"
        WARNING = "WARNING"
        ERROR = "ERROR"

    def __init__(self, version, payload, call_sid):
        """
        Initialize the EventInstance

        :returns: twilio.rest.insights.v1.call.event.EventInstance
        :rtype: twilio.rest.insights.v1.call.event.EventInstance
        """
        super(EventInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'timestamp': payload.get('timestamp'),
            'call_sid': payload.get('call_sid'),
            'account_sid': payload.get('account_sid'),
            'edge': payload.get('edge'),
            'group': payload.get('group'),
            'level': payload.get('level'),
            'name': payload.get('name'),
            'carrier_edge': payload.get('carrier_edge'),
            'sip_edge': payload.get('sip_edge'),
            'sdk_edge': payload.get('sdk_edge'),
            'client_edge': payload.get('client_edge'),
        }

        # Context
        self._context = None
        self._solution = {'call_sid': call_sid, }

    @property
    def timestamp(self):
        """
        :returns: The timestamp
        :rtype: unicode
        """
        return self._properties['timestamp']

    @property
    def call_sid(self):
        """
        :returns: The call_sid
        :rtype: unicode
        """
        return self._properties['call_sid']

    @property
    def account_sid(self):
        """
        :returns: The account_sid
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def edge(self):
        """
        :returns: The edge
        :rtype: EventInstance.TwilioEdge
        """
        return self._properties['edge']

    @property
    def group(self):
        """
        :returns: The group
        :rtype: unicode
        """
        return self._properties['group']

    @property
    def level(self):
        """
        :returns: The level
        :rtype: EventInstance.Level
        """
        return self._properties['level']

    @property
    def name(self):
        """
        :returns: The name
        :rtype: unicode
        """
        return self._properties['name']

    @property
    def carrier_edge(self):
        """
        :returns: The carrier_edge
        :rtype: dict
        """
        return self._properties['carrier_edge']

    @property
    def sip_edge(self):
        """
        :returns: The sip_edge
        :rtype: dict
        """
        return self._properties['sip_edge']

    @property
    def sdk_edge(self):
        """
        :returns: The sdk_edge
        :rtype: dict
        """
        return self._properties['sdk_edge']

    @property
    def client_edge(self):
        """
        :returns: The client_edge
        :rtype: dict
        """
        return self._properties['client_edge']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.EventInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class SimIpAddressList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, sim_sid):
        """
        Initialize the SimIpAddressList

        :param Version version: Version that contains the resource
        :param sim_sid: The unique string that identifies the resource

        :returns: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressList
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressList
        """
        super(SimIpAddressList, self).__init__(version)

        # Path Solution
        self._solution = {'sim_sid': sim_sid, }
        self._uri = '/Sims/{sim_sid}/IpAddresses'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams SimIpAddressInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists SimIpAddressInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of SimIpAddressInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SimIpAddressInstance
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return SimIpAddressPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SimIpAddressInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SimIpAddressInstance
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return SimIpAddressPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SimIpAddressList>'


class SimIpAddressPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the SimIpAddressPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param sim_sid: The unique string that identifies the resource

        :returns: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressPage
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressPage
        """
        super(SimIpAddressPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of SimIpAddressInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance
        """
        return SimIpAddressInstance(self._version, payload, sim_sid=self._solution['sim_sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SimIpAddressPage>'


class SimIpAddressInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class IpAddressVersion(object):
        IPV4 = "IPv4"
        IPV6 = "IPv6"

    def __init__(self, version, payload, sim_sid):
        """
        Initialize the SimIpAddressInstance

        :returns: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance
        :rtype: twilio.rest.supersim.v1.sim.sim_ip_address.SimIpAddressInstance
        """
        super(SimIpAddressInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'ip_address': payload.get('ip_address'),
            'ip_address_version': payload.get('ip_address_version'),
        }

        # Context
        self._context = None
        self._solution = {'sim_sid': sim_sid, }

    @property
    def ip_address(self):
        """
        :returns: IP address assigned to the given Super SIM
        :rtype: unicode
        """
        return self._properties['ip_address']

    @property
    def ip_address_version(self):
        """
        :returns: IP address version
        :rtype: SimIpAddressInstance.IpAddressVersion
        """
        return self._properties['ip_address_version']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SimIpAddressInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class PhoneNumberList(ListResource):

    def __init__(self, version):
        """
        Initialize the PhoneNumberList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberList
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberList
        """
        super(PhoneNumberList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, phone_number):
        """
        Constructs a PhoneNumberContext

        :param phone_number: The phone number

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        """
        return PhoneNumberContext(self._version, phone_number=phone_number, )

    def __call__(self, phone_number):
        """
        Constructs a PhoneNumberContext

        :param phone_number: The phone number

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        """
        return PhoneNumberContext(self._version, phone_number=phone_number, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes.V2.PhoneNumberList>'


class PhoneNumberPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the PhoneNumberPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberPage
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberPage
        """
        super(PhoneNumberPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of PhoneNumberInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        return PhoneNumberInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes.V2.PhoneNumberPage>'


class PhoneNumberContext(InstanceContext):

    def __init__(self, version, phone_number):
        """
        Initialize the PhoneNumberContext

        :param Version version: Version that contains the resource
        :param phone_number: The phone number

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        """
        super(PhoneNumberContext, self).__init__(version)

        # Path Solution
        self._solution = {'phone_number': phone_number, }
        self._uri = '/PhoneNumbers/{phone_number}'.format(**self._solution)

    def update(self, voice_region=values.unset, friendly_name=values.unset):
        """
        Update the PhoneNumberInstance

        :param unicode voice_region: The Inbound Processing Region used for this phone number for voice
        :param unicode friendly_name: A human readable description of this resource.

        :returns: The updated PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        data = values.of({'VoiceRegion': voice_region, 'FriendlyName': friendly_name, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return PhoneNumberInstance(self._version, payload, phone_number=self._solution['phone_number'], )

    def fetch(self):
        """
        Fetch the PhoneNumberInstance

        :returns: The fetched PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return PhoneNumberInstance(self._version, payload, phone_number=self._solution['phone_number'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Routes.V2.PhoneNumberContext {}>'.format(context)


class PhoneNumberInstance(InstanceResource):

    def __init__(self, version, payload, phone_number=None):
        """
        Initialize the PhoneNumberInstance

        :returns: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        super(PhoneNumberInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'phone_number': payload.get('phone_number'),
            'url': payload.get('url'),
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'voice_region': payload.get('voice_region'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
        }

        # Context
        self._context = None
        self._solution = {'phone_number': phone_number or self._properties['phone_number'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: PhoneNumberContext for this PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberContext
        """
        if self._context is None:
            self._context = PhoneNumberContext(self._version, phone_number=self._solution['phone_number'], )
        return self._context

    @property
    def phone_number(self):
        """
        :returns: The phone number
        :rtype: unicode
        """
        return self._properties['phone_number']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies the Inbound Processing Region assignments for this phone number.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: Account Sid.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: A human readable description of the Inbound Processing Region assignments for this phone number.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def voice_region(self):
        """
        :returns: The Inbound Processing Region used for this phone number for voice.
        :rtype: unicode
        """
        return self._properties['voice_region']

    @property
    def date_created(self):
        """
        :returns: The date that this phone number was assigned an Inbound Processing Region.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that the Inbound Processing Region was updated for this phone number.
        :rtype: datetime
        """
        return self._properties['date_updated']

    def update(self, voice_region=values.unset, friendly_name=values.unset):
        """
        Update the PhoneNumberInstance

        :param unicode voice_region: The Inbound Processing Region used for this phone number for voice
        :param unicode friendly_name: A human readable description of this resource.

        :returns: The updated PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        return self._proxy.update(voice_region=voice_region, friendly_name=friendly_name, )

    def fetch(self):
        """
        Fetch the PhoneNumberInstance

        :returns: The fetched PhoneNumberInstance
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Routes.V2.PhoneNumberInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.numbers.v2.regulatory_compliance.bundle import BundleList
from twilio.rest.numbers.v2.regulatory_compliance.end_user import EndUserList
from twilio.rest.numbers.v2.regulatory_compliance.end_user_type import EndUserTypeList
from twilio.rest.numbers.v2.regulatory_compliance.regulation import RegulationList
from twilio.rest.numbers.v2.regulatory_compliance.supporting_document import SupportingDocumentList
from twilio.rest.numbers.v2.regulatory_compliance.supporting_document_type import SupportingDocumentTypeList


class RegulatoryComplianceList(ListResource):

    def __init__(self, version):
        """
        Initialize the RegulatoryComplianceList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceList
        """
        super(RegulatoryComplianceList, self).__init__(version)

        # Path Solution
        self._solution = {}

        # Components
        self._bundles = None
        self._end_users = None
        self._end_user_types = None
        self._regulations = None
        self._supporting_documents = None
        self._supporting_document_types = None

    @property
    def bundles(self):
        """
        Access the bundles

        :returns: twilio.rest.numbers.v2.regulatory_compliance.bundle.BundleList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.bundle.BundleList
        """
        if self._bundles is None:
            self._bundles = BundleList(self._version, )
        return self._bundles

    @property
    def end_users(self):
        """
        Access the end_users

        :returns: twilio.rest.numbers.v2.regulatory_compliance.end_user.EndUserList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.end_user.EndUserList
        """
        if self._end_users is None:
            self._end_users = EndUserList(self._version, )
        return self._end_users

    @property
    def end_user_types(self):
        """
        Access the end_user_types

        :returns: twilio.rest.numbers.v2.regulatory_compliance.end_user_type.EndUserTypeList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.end_user_type.EndUserTypeList
        """
        if self._end_user_types is None:
            self._end_user_types = EndUserTypeList(self._version, )
        return self._end_user_types

    @property
    def regulations(self):
        """
        Access the regulations

        :returns: twilio.rest.numbers.v2.regulatory_compliance.regulation.RegulationList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.regulation.RegulationList
        """
        if self._regulations is None:
            self._regulations = RegulationList(self._version, )
        return self._regulations

    @property
    def supporting_documents(self):
        """
        Access the supporting_documents

        :returns: twilio.rest.numbers.v2.regulatory_compliance.supporting_document.SupportingDocumentList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.supporting_document.SupportingDocumentList
        """
        if self._supporting_documents is None:
            self._supporting_documents = SupportingDocumentList(self._version, )
        return self._supporting_documents

    @property
    def supporting_document_types(self):
        """
        Access the supporting_document_types

        :returns: twilio.rest.numbers.v2.regulatory_compliance.supporting_document_type.SupportingDocumentTypeList
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.supporting_document_type.SupportingDocumentTypeList
        """
        if self._supporting_document_types is None:
            self._supporting_document_types = SupportingDocumentTypeList(self._version, )
        return self._supporting_document_types

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Numbers.V2.RegulatoryComplianceList>'


class RegulatoryCompliancePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the RegulatoryCompliancePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryCompliancePage
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryCompliancePage
        """
        super(RegulatoryCompliancePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of RegulatoryComplianceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceInstance
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceInstance
        """
        return RegulatoryComplianceInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Numbers.V2.RegulatoryCompliancePage>'


class RegulatoryComplianceInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the RegulatoryComplianceInstance

        :returns: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceInstance
        :rtype: twilio.rest.numbers.v2.regulatory_compliance.RegulatoryComplianceInstance
        """
        super(RegulatoryComplianceInstance, self).__init__(version)

        # Context
        self._context = None
        self._solution = {}

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Numbers.V2.RegulatoryComplianceInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.messaging.v1.brand_registration import BrandRegistrationList
from twilio.rest.messaging.v1.deactivation import DeactivationsList
from twilio.rest.messaging.v1.external_campaign import ExternalCampaignList
from twilio.rest.messaging.v1.service import ServiceList
from twilio.rest.messaging.v1.tollfree_verification import TollfreeVerificationList
from twilio.rest.messaging.v1.usecase import UsecaseList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Messaging

        :returns: V1 version of Messaging
        :rtype: twilio.rest.messaging.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._brand_registrations = None
        self._deactivations = None
        self._external_campaign = None
        self._services = None
        self._tollfree_verifications = None
        self._usecases = None

    @property
    def brand_registrations(self):
        """
        :rtype: twilio.rest.messaging.v1.brand_registration.BrandRegistrationList
        """
        if self._brand_registrations is None:
            self._brand_registrations = BrandRegistrationList(self)
        return self._brand_registrations

    @property
    def deactivations(self):
        """
        :rtype: twilio.rest.messaging.v1.deactivation.DeactivationsList
        """
        if self._deactivations is None:
            self._deactivations = DeactivationsList(self)
        return self._deactivations

    @property
    def external_campaign(self):
        """
        :rtype: twilio.rest.messaging.v1.external_campaign.ExternalCampaignList
        """
        if self._external_campaign is None:
            self._external_campaign = ExternalCampaignList(self)
        return self._external_campaign

    @property
    def services(self):
        """
        :rtype: twilio.rest.messaging.v1.service.ServiceList
        """
        if self._services is None:
            self._services = ServiceList(self)
        return self._services

    @property
    def tollfree_verifications(self):
        """
        :rtype: twilio.rest.messaging.v1.tollfree_verification.TollfreeVerificationList
        """
        if self._tollfree_verifications is None:
            self._tollfree_verifications = TollfreeVerificationList(self)
        return self._tollfree_verifications

    @property
    def usecases(self):
        """
        :rtype: twilio.rest.messaging.v1.usecase.UsecaseList
        """
        if self._usecases is None:
            self._usecases = UsecaseList(self)
        return self._usecases

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Messaging.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.verify.v2.form import FormList
from twilio.rest.verify.v2.safelist import SafelistList
from twilio.rest.verify.v2.service import ServiceList
from twilio.rest.verify.v2.template import TemplateList
from twilio.rest.verify.v2.verification_attempt import VerificationAttemptList
from twilio.rest.verify.v2.verification_attempts_summary import VerificationAttemptsSummaryList


class V2(Version):

    def __init__(self, domain):
        """
        Initialize the V2 version of Verify

        :returns: V2 version of Verify
        :rtype: twilio.rest.verify.v2.V2.V2
        """
        super(V2, self).__init__(domain)
        self.version = 'v2'
        self._forms = None
        self._safelist = None
        self._services = None
        self._verification_attempts = None
        self._verification_attempts_summary = None
        self._templates = None

    @property
    def forms(self):
        """
        :rtype: twilio.rest.verify.v2.form.FormList
        """
        if self._forms is None:
            self._forms = FormList(self)
        return self._forms

    @property
    def safelist(self):
        """
        :rtype: twilio.rest.verify.v2.safelist.SafelistList
        """
        if self._safelist is None:
            self._safelist = SafelistList(self)
        return self._safelist

    @property
    def services(self):
        """
        :rtype: twilio.rest.verify.v2.service.ServiceList
        """
        if self._services is None:
            self._services = ServiceList(self)
        return self._services

    @property
    def verification_attempts(self):
        """
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptList
        """
        if self._verification_attempts is None:
            self._verification_attempts = VerificationAttemptList(self)
        return self._verification_attempts

    @property
    def verification_attempts_summary(self):
        """
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryList
        """
        if self._verification_attempts_summary is None:
            self._verification_attempts_summary = VerificationAttemptsSummaryList(self)
        return self._verification_attempts_summary

    @property
    def templates(self):
        """
        :rtype: twilio.rest.verify.v2.template.TemplateList
        """
        if self._templates is None:
            self._templates = TemplateList(self)
        return self._templates

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ArchivedCallList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the ArchivedCallList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallList
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallList
        """
        super(ArchivedCallList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, date, sid):
        """
        Constructs a ArchivedCallContext

        :param date: The date of the Call in UTC.
        :param sid: The unique string that identifies this resource

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        """
        return ArchivedCallContext(self._version, date=date, sid=sid, )

    def __call__(self, date, sid):
        """
        Constructs a ArchivedCallContext

        :param date: The date of the Call in UTC.
        :param sid: The unique string that identifies this resource

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        """
        return ArchivedCallContext(self._version, date=date, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.ArchivedCallList>'


class ArchivedCallPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the ArchivedCallPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallPage
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallPage
        """
        super(ArchivedCallPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ArchivedCallInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallInstance
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallInstance
        """
        return ArchivedCallInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Voice.V1.ArchivedCallPage>'


class ArchivedCallContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, date, sid):
        """
        Initialize the ArchivedCallContext

        :param Version version: Version that contains the resource
        :param date: The date of the Call in UTC.
        :param sid: The unique string that identifies this resource

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        """
        super(ArchivedCallContext, self).__init__(version)

        # Path Solution
        self._solution = {'date': date, 'sid': sid, }
        self._uri = '/Archives/{date}/Calls/{sid}'.format(**self._solution)

    def delete(self):
        """
        Deletes the ArchivedCallInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Voice.V1.ArchivedCallContext {}>'.format(context)


class ArchivedCallInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, payload, date=None, sid=None):
        """
        Initialize the ArchivedCallInstance

        :returns: twilio.rest.voice.v1.archived_call.ArchivedCallInstance
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallInstance
        """
        super(ArchivedCallInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'date': deserialize.iso8601_date(payload.get('date')),
            'sid': payload.get('sid'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'date': date or self._properties['date'], 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ArchivedCallContext for this ArchivedCallInstance
        :rtype: twilio.rest.voice.v1.archived_call.ArchivedCallContext
        """
        if self._context is None:
            self._context = ArchivedCallContext(
                self._version,
                date=self._solution['date'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def date(self):
        """
        :returns: date
        :rtype: date
        """
        return self._properties['date']

    @property
    def sid(self):
        """
        :returns: sid
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    def delete(self):
        """
        Deletes the ArchivedCallInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Voice.V1.ArchivedCallInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.oauth.v1.oauth import OauthList
from twilio.rest.oauth.v1.openid_discovery import OpenidDiscoveryList
from twilio.rest.oauth.v1.token import TokenList
from twilio.rest.oauth.v1.user_info import UserInfoList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Oauth

        :returns: V1 version of Oauth
        :rtype: twilio.rest.oauth.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._oauth = None
        self._openid_discovery = None
        self._token = None
        self._user_info = None

    @property
    def oauth(self):
        """
        :rtype: twilio.rest.oauth.v1.oauth.OauthList
        """
        if self._oauth is None:
            self._oauth = OauthList(self)
        return self._oauth

    @property
    def openid_discovery(self):
        """
        :rtype: twilio.rest.oauth.v1.openid_discovery.OpenidDiscoveryList
        """
        if self._openid_discovery is None:
            self._openid_discovery = OpenidDiscoveryList(self)
        return self._openid_discovery

    @property
    def token(self):
        """
        :rtype: twilio.rest.oauth.v1.token.TokenList
        """
        if self._token is None:
            self._token = TokenList(self)
        return self._token

    @property
    def user_info(self):
        """
        :rtype: twilio.rest.oauth.v1.user_info.UserInfoList
        """
        if self._user_info is None:
            self._user_info = UserInfoList(self)
        return self._user_info

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1>'
